@@ -35,6 +35,8 @@
 #include <QIcon>
 #include <QTimer>
 
+#include <algorithm>
+
 Notification::Notification(QObject *parent) :
     QObject(parent)
 {
@@ -78,8 +80,7 @@ void Notification::stateChanged(NetworkManager::Device::State newstate, NetworkM
     NetworkManager::Device *device = qobject_cast<NetworkManager::Device*>(sender());
     if (newstate == NetworkManager::Device::Activated && m_notifications.contains(device->uni())) {
         KNotification *notify = m_notifications.value(device->uni());
-        notify->deleteLater();
-        m_notifications.remove(device->uni());
+        notify->close();
         return;
     } else if (newstate != NetworkManager::Device::Failed) {
         return;
@@ -331,17 +332,15 @@ void Notification::stateChanged(NetworkManager::Device::State newstate, NetworkM
         notify->setText(text.toHtmlEscaped());
         notify->update();
     } else {
-        KNotification *notify = new KNotification(QStringLiteral("DeviceFailed"), KNotification::CloseOnTimeout, this);
+        KNotification *notify = new KNotification(QStringLiteral("DeviceFailed"), KNotification::CloseOnTimeout);
         connect(notify, &KNotification::closed, this, &Notification::notificationClosed);
         notify->setProperty("uni", device->uni());
         notify->setComponentName(QStringLiteral("networkmanagement"));
         notify->setIconName(QStringLiteral("dialog-warning"));
         notify->setTitle(identifier);
         notify->setText(text.toHtmlEscaped());
+        m_notifications[device->uni()] = notify;
         notify->sendEvent();
-        if (notify->id() != -1) {
-                m_notifications[device->uni()] = notify;
-        }
     }
 }
 
@@ -375,7 +374,7 @@ void Notification::onActiveConnectionStateChanged(NetworkManager::ActiveConnecti
 
     QString eventId, text, iconName;
     const QString acName = ac->id();
-    const QString connectionId = ac->path();
+    const QString connectionId = ac->id();
 
     if (state == NetworkManager::ActiveConnection::Activated) {
         auto foundConnection = std::find_if(m_activeConnectionsBeforeSleep.constBegin(),
@@ -429,10 +428,16 @@ void Notification::onActiveConnectionStateChanged(NetworkManager::ActiveConnecti
         return;
     }
 
-    KNotification *notify = new KNotification(eventId, KNotification::CloseOnTimeout, this);
-    connect(notify, &KNotification::closed, this, &Notification::notificationClosed);
-    notify->setProperty("uni", connectionId);
-    notify->setComponentName(QStringLiteral("networkmanagement"));
+    KNotification *notify = m_notifications.value(connectionId);
+
+    if (!notify) {
+        notify = new KNotification(eventId, KNotification::CloseOnTimeout);
+        connect(notify, &KNotification::closed, this, &Notification::notificationClosed);
+        notify->setProperty("uni", connectionId);
+        notify->setComponentName(QStringLiteral("networkmanagement"));
+        m_notifications[connectionId] = notify;
+    }
+
     if (!iconName.isEmpty()) {
         notify->setIconName(iconName);
     } else {
@@ -445,9 +450,6 @@ void Notification::onActiveConnectionStateChanged(NetworkManager::ActiveConnecti
     notify->setTitle(acName);
     notify->setText(text.toHtmlEscaped());
     notify->sendEvent();
-    if (notify->id() != -1) {
-        m_notifications[connectionId] = notify;
-    }
 }
 
 void Notification::onVpnConnectionStateChanged(NetworkManager::VpnConnection::State state, NetworkManager::VpnConnection::StateChangeReason reason)
@@ -509,7 +511,7 @@ void Notification::onVpnConnectionStateChanged(NetworkManager::VpnConnection::St
         break;
     }
 
-    KNotification *notify = new KNotification(eventId, KNotification::CloseOnTimeout, this);
+    KNotification *notify = new KNotification(eventId, KNotification::CloseOnTimeout);
     connect(notify, &KNotification::closed, this, &Notification::notificationClosed);
     notify->setProperty("uni", connectionId);
     notify->setComponentName("networkmanagement");
@@ -520,17 +522,14 @@ void Notification::onVpnConnectionStateChanged(NetworkManager::VpnConnection::St
     }
     notify->setTitle(vpnName);
     notify->setText(text.toHtmlEscaped());
+    m_notifications[connectionId] = notify;
     notify->sendEvent();
-    if (notify->id() != -1) {
-        m_notifications[connectionId] = notify;
-    }
 }
 
 void Notification::notificationClosed()
 {
     KNotification *notify = qobject_cast<KNotification*>(sender());
     m_notifications.remove(notify->property("uni").toString());
-    notify->deleteLater();
 }
 
 void Notification::onPrepareForSleep(bool sleep)
@@ -572,16 +571,16 @@ void Notification::onCheckActiveConnectionOnResume()
 
     m_activeConnectionsBeforeSleep.clear();
 
-    const auto &connections = NetworkManager::activeConnections();
-    for (const auto &connection : connections) {
-        if (connection->state() == NetworkManager::ActiveConnection::State::Activated ||
-            connection->state() == NetworkManager::ActiveConnection::State::Activating) {
-            // we have an active or activating connection, don't tell the user we're no longer connected
-            return;
-        }
+    const auto ac = NetworkManager::activeConnections();
+    if (std::any_of(ac.constBegin(), ac.constEnd(), [](const auto &connection) {
+       return connection->state() == NetworkManager::ActiveConnection::State::Activated
+           || connection->state() == NetworkManager::ActiveConnection::State::Activating;
+    })) {
+        // we have an active or activating connection, don't tell the user we're no longer connected
+        return;
     }
 
-    KNotification *notify = new KNotification(QStringLiteral("NoLongerConnected"), KNotification::CloseOnTimeout, this);
+    KNotification *notify = new KNotification(QStringLiteral("NoLongerConnected"), KNotification::CloseOnTimeout);
     connect(notify, &KNotification::closed, this, &Notification::notificationClosed);
     const QString uni = QStringLiteral("offlineNotification");
     notify->setProperty("uni", uni);
@@ -589,8 +588,6 @@ void Notification::onCheckActiveConnectionOnResume()
     notify->setIconName(QStringLiteral("dialog-warning"));
     notify->setTitle(i18n("No Network Connection"));
     notify->setText(i18n("You are no longer connected to a network."));
+    m_notifications[uni] = notify;
     notify->sendEvent();
-    if (notify->id() != -1) {
-        m_notifications[uni] = notify;
-    }
 }
